package com.sundata.internalevaluation.calc.running.threads;

import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import org.jgrapht.Graph;
import org.jgrapht.graph.DefaultEdge;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.RecursiveTask;

/**
 * 分解用的任务类，结果有 ClacResult 组成
 */
public class CalcRecursiveTask extends RecursiveTask<CalcResult<String, Object>> {

    private static final Logger log = LoggerFactory.getLogger(CalcRecursiveTask.class);

    private final String calculateInstanceNumber;
    private final CalcUnit calcUnit;
    private final ConcurrentHashMap<String, Object> content;
    private final Graph<CalcUnit, DefaultEdge> graph;
    private final ConcurrentHashMap<CalcUnit, CalcResult<String, Object>> results;
    private ConcurrentHashMap<CalcUnit, CalcResult<String, Object>> childResults;

    public CalcRecursiveTask(String calculateInstanceNumber, CalcUnit calcUnit, ConcurrentHashMap<String, Object> content, Graph<CalcUnit, DefaultEdge> graph, ConcurrentHashMap<CalcUnit, CalcResult<String, Object>> results, ConcurrentHashMap<CalcUnit, CalcResult<String, Object>> childResults) {
        this.calculateInstanceNumber = calculateInstanceNumber;
        this.calcUnit = calcUnit;
        this.content = content;
        this.graph = graph;
        this.results = results;
        this.childResults = childResults;
    }

    /**
     * The main computation performed by this task.
     *
     * @return the result of the computation
     */
    @Override
    protected CalcResult<String, Object> compute() {
        log.debug("开始计算");
        if (results.containsKey(calcUnit)) {
            log.debug("已经计算过，不再计算");
            return results.get(calcUnit);
        }
        List<CalcUnit> predecessors = graph.incomingEdgesOf(calcUnit)
                .stream()
                .map(graph::getEdgeSource)
                .toList();
        // 如果有来源节点，则去计算来源节点并合并计算内容，如果没有来源节点，则处理
        if (!predecessors.isEmpty()) {
            childResults = new ConcurrentHashMap<>();
            List<CalcRecursiveTask> tasks = predecessors.stream()
                    .map(predecessor -> new CalcRecursiveTask(calculateInstanceNumber, predecessor, content, graph,results,childResults))
                    .toList();
            for (int i = 0; i < predecessors.size(); i++) {
                CalcRecursiveTask task = tasks.get(i);
//                task.fork();
                childResults.put(predecessors.get(i), task.invoke());
            }
        }
        calcUnit.startCalc(calculateInstanceNumber, content,childResults);// 假设节点名称是一个整数
        CalcResult<String, Object> result = calcUnit.getResultContext();
        results.put(calcUnit, result);
        log.debug("计算完成");
        return result;
    }


    public Map<String, Object> getContent() {
        return content;
    }

    public String getCalculateInstanceNumber() {
        return calculateInstanceNumber;
    }
}
