package com.sundata.internalevaluation.configuration.service;

import com.sundata.common.exception.BusinessException;
import com.sundata.internalevaluation.configuration.mapper.SysReqRulesMapper;
import com.sundata.internalevaluation.configuration.model.SysReqRule;
import com.sundata.internalevaluation.configuration.mapper.SysReqRuleMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class SysReqRuleService {

    private static final Logger log = LoggerFactory.getLogger(SysReqRuleService.class);
    @Autowired
    private SysReqRuleMapper sysReqRuleMapper;

    @Autowired
    private SysReqRulesMapper sysReqRulesMapper;

    /**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
    public List<SysReqRule> listAll(SysReqRule sysReqRule) {
    	return sysReqRuleMapper.listAll(sysReqRule);
    }


    /**
     * 根据主键查询
     *
     * @param ruleNo 主键
     * @return 返回记录，没有返回null
     */
    public SysReqRule getById(String ruleNo) {
    	return sysReqRuleMapper.getById(ruleNo);
    }
	

    /**
     * 新增，忽略null字段
     *
     * @param sysReqRule 新增的记录
     * @return 返回影响行数
     */
    public int insertIgnoreNull(SysReqRule sysReqRule) {
        try {
            return sysReqRuleMapper.insertIgnoreNull(sysReqRule);
        } catch (DuplicateKeyException e) {
            throw new BusinessException("规则编码已存在，请重新输入.");
        }
    }
	


    /**
     * 修改，忽略null字段
     *
     * @param sysReqRule 修改的记录
     * @return 返回影响行数
     */
    public int updateIgnoreNull(SysReqRule sysReqRule) {
    	return sysReqRuleMapper.updateIgnoreNull(sysReqRule);
    }
	
    /**
     * 删除记录
     *
     * @param sysReqRule 待删除的记录
     * @return 返回影响行数
     */
    public int delete(SysReqRule sysReqRule) {


        // 查询是否再其他规则集中引用
        List<String> rulesNo = sysReqRulesMapper.selectRulesByRuleNo(sysReqRule.getRuleNo());

        if (rulesNo == null || rulesNo.isEmpty()) {
            return sysReqRuleMapper.delete(sysReqRule);
        } else {
            throw new BusinessException("该规则已被引用，不能删除");
//            log.error("该规则已被引用，不能删除");
//            return 0;
        }
    }
	
}